==> glib-web-0.5.86.tgz: package/components/mixins/ws/phoenixSocket.js <==
import { Socket } from "phoenix";
import Hash from "../../../utils/hash";

export default {
  data: function() {
    return {
      _wsSocket: null
    };
  },
  methods: {
    $wsInitPhoenixSocket(spec) {
      Utils.type.ifObject(
        spec,
        ws => {
          console.log("Connecting socket...");
          Utils.launch.snackbar.indicator("Connecting...");
          Utils.http.startIndicator(this);

          // Without this, at times the indicator doesn't disappear because there isn't enough delay
          // between startIndicator() and stopIndicator()
          setTimeout(() => {
            this._wsConnectSocket(ws);
          }, 100);
        },
        () => {
          this._wsDisconnectSocket();
        }
      );
    },
    // https://medium.com/@imiked/real-time-feature-with-phoenix-and-vue-js-c7677663a6f5
    _wsConnectSocket(spec) {
      const host = Utils.settings.isDev
        ? "ws://localhost:4019"
        : "wss://prod.com";

      // const host = Utils.settings.isDev
      //   ? "ws://192.168.1.109:4019"
      //   : "wss://prod.com";

      // const url = Utils.url.appendParams(
      //   `${host}${conf.endpoint}?`,
      //   conf.params
      // );

      const conf = spec.socket;
      // See https://github.com/phoenixframework/phoenix/blob/master/assets/js/phoenix.js
      const endpoint = `${host}${conf.endpoint}`.replace(/\/websocket$/, "");
      const params = new Hash(conf.params);
      const version = params.remove("vsn");
      const socket = new Socket(endpoint, {
        params: params,
        vsn: version
      });

      this._wsSocket = socket;
      socket.onOpen(() => {
        console.log("Connected socket");
        this._wsJoinChannel(socket, spec);
        Utils.http.stopIndicator(this);
      });
      socket.onError(() => {
        // Start the indicator again because this can trigger even after successfully connected.
        Utils.http.startIndicator(this);
        console.log("Unable to connect socket");
      });
      socket.connect();
    },
    _wsJoinChannel(socket, spec) {
      const ws = window.vueApp.webSocket;
      Utils.type.ifObject(spec.header, header => {
        ws.header = header;
      });

      const topicName = spec.topic;
      const events = spec.events;
      if (!Utils.type.isString(topicName) || !Utils.type.isArray(events)) {
        return;
      }

      const channel = socket.channel(topicName, {});

      for (const eventName of events) {
        console.log(`Registering event '${eventName}'`);
        channel.on(eventName, payload => {
          console.debug(`Received '${eventName}' event`, payload);
          Utils.ws.handleResponse(payload.onResponse, this);
        });
      }

      channel
        .join()
        .receive("ok", resp => {
          console.log(`Channel joined '${topicName}'`, resp);
          ws.channels[topicName] = channel;
        })
        .receive("error", resp => {
          console.log(`Unable to join channel '${topicName}'`, resp);
          Utils.launch.snackbar.error("Unable to connect. Please try again.");
        });
    },
    _wsDisconnectSocket() {
      if (this._wsSocket) {
        console.log("Disconnecting socket");
        // Sometimes this doesn't prevent errorneous socket from trying to reconnect. In this case,
        // the socket will keep attempting to connect.
        this._wsSocket.disconnect();
        this._wsSocket = null;

        const ws = window.vueApp.webSocket;
        ws.channels = {};
        ws.header = {};

        Utils.http.stopIndicator(this);
      }
    }
  }
};

==> glib-web-0.11.8.tgz: package/components/mixins/ws/phoenixSocket.js <==
import { Socket } from "phoenix";
import Hash from "../../../utils/hash";

export default {
  data: function() {
    return {
      _wsSocket: null
    };
  },
  methods: {
    $wsInitPhoenixSocket(spec) {
      Utils.type.ifObject(
        spec,
        ws => {
          console.log("Connecting socket...");
          Utils.launch.snackbar.indicator("Connecting...");
          Utils.http.startIndicator(this);

          // Without this, at times the indicator doesn't disappear because there isn't enough delay
          // between startIndicator() and stopIndicator()
          setTimeout(() => {
            this._wsConnectSocket(ws);
          }, 100);
        },
        () => {
          this._wsDisconnectSocket();
        }
      );
    },
    // https://medium.com/@imiked/real-time-feature-with-phoenix-and-vue-js-c7677663a6f5
    _wsConnectSocket(spec) {
      const host = Utils.settings.isDev
        ? "ws://localhost:4019"
        : "wss://prod.com";

      // const host = Utils.settings.isDev
      //   ? "ws://192.168.1.109:4019"
      //   : "wss://prod.com";

      // const url = Utils.url.appendParams(
      //   `${host}${conf.endpoint}?`,
      //   conf.params
      // );

      const conf = spec.socket;
      // See https://github.com/phoenixframework/phoenix/blob/master/assets/js/phoenix.js
      const endpoint = `${host}${conf.endpoint}`.replace(/\/websocket$/, "");
      const params = new Hash(conf.params);
      const version = params.remove("vsn");
      const socket = new Socket(endpoint, {
        params: params,
        vsn: version
      });

      this._wsSocket = socket;
      socket.onOpen(() => {
        console.log("Connected socket");
        this._wsJoinChannel(socket, spec);
        Utils.http.stopIndicator(this);
      });
      socket.onError(() => {
        // Start the indicator again because this can trigger even after successfully connected.
        Utils.http.startIndicator(this);
        console.log("Unable to connect socket");
      });
      socket.connect();
    },
    _wsJoinChannel(socket, spec) {
      const ws = window.vueApp.webSocket;
      Utils.type.ifObject(spec.header, header => {
        ws.header = header;
      });

      const topicName = spec.topic;
      const events = spec.events;
      if (!Utils.type.isString(topicName) || !Utils.type.isArray(events)) {
        return;
      }

      const channel = socket.channel(topicName, {});

      for (const eventName of events) {
        console.log(`Registering event '${eventName}'`);
        channel.on(eventName, payload => {
          console.debug(`Received '${eventName}' event`, payload);
          Utils.ws.handleResponse(payload.onResponse, this);
        });
      }

      channel
        .join()
        .receive("ok", resp => {
          console.log(`Channel joined '${topicName}'`, resp);
          ws.channels[topicName] = channel;
        })
        .receive("error", resp => {
          console.log(`Unable to join channel '${topicName}'`, resp);
          Utils.launch.snackbar.error(
            "Unable to connect. Please try again.",
            this
          );
        });
    },
    _wsDisconnectSocket() {
      if (this._wsSocket) {
        console.log("Disconnecting socket");
        // Sometimes this doesn't prevent errorneous socket from trying to reconnect. In this case,
        // the socket will keep attempting to connect.
        this._wsSocket.disconnect();
        this._wsSocket = null;

        const ws = window.vueApp.webSocket;
        ws.channels = {};
        ws.header = {};

        Utils.http.stopIndicator(this);
      }
    }
  }
};
